var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createElement as _createElement } from "react";
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useState, useId, useCallback, useRef, useImperativeHandle, useEffect, useMemo } from 'react';
import ReactDOM from 'react-dom';
import { useAsync, useElement, useEventCallback } from '@react-devui/hooks';
import { LoadingOutlined } from '@react-devui/icons';
import { findNested, getClassName, getOriginalSize, getVerticalSidePosition } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useTranslation, useMaxIndex, useDValue, useLayout } from '../../hooks';
import { registerComponentMate, TTANSITION_DURING_POPUP } from '../../utils';
import { DFocusVisible } from '../_focus-visible';
import { DComboboxKeyboardSupport } from '../_keyboard-support';
import { DTransition } from '../_transition';
import { DInput } from '../input';
import { DVirtualScroll } from '../virtual-scroll';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DAutoComplete' });
function AutoComplete(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { children, dList, dVisible, dLoading = false, dCustomItem, onVisibleChange, afterVisibleChange, onItemClick, onScrollBottom } = _a, restProps = __rest(_a, ["children", "dList", "dVisible", "dLoading", "dCustomItem", "onVisibleChange", "afterVisibleChange", "onItemClick", "onScrollBottom"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dScrollEl, dResizeEl } = useLayout();
    //#endregion
    //#region Ref
    const popupRef = useRef(null);
    const dVSRef = useRef(null);
    //#endregion
    const [t] = useTranslation();
    const asyncCapture = useAsync();
    const uniqueId = useId();
    const listId = `${dPrefix}auto-complete-list-${uniqueId}`;
    const getGroupId = (val) => `${dPrefix}auto-complete-group-${val}-${uniqueId}`;
    const getItemId = (val) => `${dPrefix}auto-complete-item-${val}-${uniqueId}`;
    const scrollEl = useElement(dScrollEl);
    const resizeEl = useElement(dResizeEl);
    const containerEl = useElement(() => {
        let el = document.getElementById(`${dPrefix}auto-complete-root`);
        if (!el) {
            el = document.createElement('div');
            el.id = `${dPrefix}auto-complete-root`;
            document.body.appendChild(el);
        }
        return el;
    });
    const canSelectItem = useCallback((item) => !item.disabled && !item.children, []);
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const [focusVisible, setFocusVisible] = useState(false);
    const maxZIndex = useMaxIndex(visible);
    const [popupPositionStyle, setPopupPositionStyle] = useState({
        top: -9999,
        left: -9999,
    });
    const [transformOrigin, setTransformOrigin] = useState();
    const updatePosition = useEventCallback(() => {
        const boxEl = document.querySelector(`[data-auto-complete-boxid="${uniqueId}"]`);
        const popupEl = popupRef.current;
        if (boxEl && popupEl) {
            const width = boxEl.getBoundingClientRect().width;
            const { height } = getOriginalSize(popupEl);
            const { top, left, transformOrigin } = getVerticalSidePosition(boxEl, { width, height }, 'bottom-left', 8);
            setPopupPositionStyle({
                top,
                left,
                minWidth: width,
                maxWidth: window.innerWidth - left - 20,
            });
            setTransformOrigin(transformOrigin);
        }
    });
    const [_focusItem, setFocusItem] = useState();
    const focusItem = (() => {
        if (_focusItem && findNested(dList, (item) => canSelectItem(item) && item.value === _focusItem.value)) {
            return _focusItem;
        }
        return findNested(dList, (item) => canSelectItem(item));
    })();
    const changeFocusItem = (item) => {
        if (!isUndefined(item)) {
            setFocusItem(item);
        }
    };
    useEffect(() => {
        if (visible) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            const boxEl = document.querySelector(`[data-auto-complete-boxid="${uniqueId}"]`);
            if (boxEl) {
                asyncGroup.onResize(boxEl, () => {
                    updatePosition();
                });
            }
            if (popupRef.current) {
                asyncGroup.onResize(popupRef.current, () => {
                    updatePosition();
                });
            }
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, uniqueId, updatePosition, visible]);
    useEffect(() => {
        if (visible && scrollEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.fromEvent(scrollEl, 'scroll', { passive: true }).subscribe({
                next: () => {
                    updatePosition();
                },
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, visible, scrollEl, updatePosition]);
    useEffect(() => {
        if (visible && resizeEl) {
            const [asyncGroup, asyncId] = asyncCapture.createGroup();
            asyncGroup.onResize(resizeEl, () => {
                updatePosition();
            });
            return () => {
                asyncCapture.deleteGroup(asyncId);
            };
        }
    }, [asyncCapture, visible, resizeEl, updatePosition]);
    const preventBlur = (e) => {
        if (e.button === 0) {
            e.preventDefault();
        }
    };
    useImperativeHandle(ref, () => ({
        updatePosition,
    }), [updatePosition]);
    const child = React.Children.only(children);
    const getInputProps = (fvProps, ksProps, inputProps) => ({
        role: 'combobox',
        'aria-autocomplete': 'list',
        'aria-expanded': visible,
        'aria-controls': listId,
        onFocus: (e) => {
            var _a;
            (_a = inputProps === null || inputProps === void 0 ? void 0 : inputProps.onFocus) === null || _a === void 0 ? void 0 : _a.call(inputProps, e);
            fvProps.fvOnFocus(e);
        },
        onBlur: (e) => {
            var _a;
            (_a = inputProps === null || inputProps === void 0 ? void 0 : inputProps.onBlur) === null || _a === void 0 ? void 0 : _a.call(inputProps, e);
            fvProps.fvOnBlur(e);
            changeVisible(false);
        },
        onClick: (e) => {
            var _a;
            (_a = inputProps === null || inputProps === void 0 ? void 0 : inputProps.onClick) === null || _a === void 0 ? void 0 : _a.call(inputProps, e);
            changeVisible(true);
        },
        onKeyDown: (e) => {
            var _a;
            (_a = inputProps === null || inputProps === void 0 ? void 0 : inputProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(inputProps, e);
            fvProps.fvOnKeyDown(e);
            ksProps.ksOnKeyDown(e);
            if (e.code === 'Enter' && visible && focusItem) {
                changeVisible(false);
                onItemClick === null || onItemClick === void 0 ? void 0 : onItemClick(focusItem.value, focusItem);
            }
        },
    });
    const vsPerformance = useMemo(() => ({
        dList,
        dItemSize: 32,
        dItemNested: (item) => ({
            list: item.children,
            emptySize: 32,
            asItem: false,
        }),
        dItemKey: (item) => item.value,
        dFocusable: canSelectItem,
    }), [canSelectItem, dList]);
    return (_jsxs(_Fragment, { children: [_jsx(DFocusVisible, Object.assign({ onFocusVisibleChange: setFocusVisible }, { children: (fvProps) => (_jsx(DComboboxKeyboardSupport, Object.assign({ dVisible: visible, dEditable: true, onVisibleChange: changeVisible, onFocusChange: (key) => {
                        var _a, _b, _c, _d;
                        switch (key) {
                            case 'next':
                                changeFocusItem((_a = dVSRef.current) === null || _a === void 0 ? void 0 : _a.scrollByStep(1));
                                break;
                            case 'prev':
                                changeFocusItem((_b = dVSRef.current) === null || _b === void 0 ? void 0 : _b.scrollByStep(-1));
                                break;
                            case 'first':
                                changeFocusItem((_c = dVSRef.current) === null || _c === void 0 ? void 0 : _c.scrollToStart());
                                break;
                            case 'last':
                                changeFocusItem((_d = dVSRef.current) === null || _d === void 0 ? void 0 : _d.scrollToEnd());
                                break;
                            default:
                                break;
                        }
                    } }, { children: (ksProps) => {
                        const childProps = Object.assign(Object.assign({}, child.props), { 'data-auto-complete-boxid': uniqueId });
                        if (child.type === DInput) {
                            childProps.dInputProps = getInputProps(fvProps, ksProps, child.props.dInputProps);
                        }
                        else {
                            Object.assign(childProps, getInputProps(fvProps, ksProps, child.props));
                        }
                        return React.cloneElement(child, childProps);
                    } }))) })), containerEl &&
                ReactDOM.createPortal(_jsx(DTransition, Object.assign({ dIn: visible, dDuring: TTANSITION_DURING_POPUP, onEnterRendered: updatePosition, afterEnter: () => {
                        afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(true);
                    }, afterLeave: () => {
                        afterVisibleChange === null || afterVisibleChange === void 0 ? void 0 : afterVisibleChange(false);
                    } }, { children: (state) => {
                        let transitionStyle = {};
                        switch (state) {
                            case 'enter':
                                transitionStyle = { transform: 'scaleY(0.7)', opacity: 0 };
                                break;
                            case 'entering':
                                transitionStyle = {
                                    transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-out`).join(', '),
                                    transformOrigin,
                                };
                                break;
                            case 'leaving':
                                transitionStyle = {
                                    transform: 'scaleY(0.7)',
                                    opacity: 0,
                                    transition: ['transform', 'opacity'].map((attr) => `${attr} ${TTANSITION_DURING_POPUP}ms ease-in`).join(', '),
                                    transformOrigin,
                                };
                                break;
                            case 'leaved':
                                transitionStyle = { display: 'none' };
                                break;
                            default:
                                break;
                        }
                        return (_jsxs("div", Object.assign({}, restProps, { ref: popupRef, className: getClassName(restProps.className, `${dPrefix}auto-complete`), style: Object.assign(Object.assign(Object.assign(Object.assign({}, restProps.style), popupPositionStyle), transitionStyle), { zIndex: maxZIndex }), onMouseDown: (e) => {
                                var _a;
                                (_a = restProps.onMouseDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                preventBlur(e);
                            }, onMouseUp: (e) => {
                                var _a;
                                (_a = restProps.onMouseUp) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                                preventBlur(e);
                            } }, { children: [dLoading && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}auto-complete__loading`, {
                                        [`${dPrefix}auto-complete__loading--empty`]: dList.length === 0,
                                    }) }, { children: _jsx(LoadingOutlined, { dSize: dList.length === 0 ? 18 : 24, dSpin: true }) }))), _jsx(DVirtualScroll, Object.assign({}, vsPerformance, { ref: dVSRef, dFillNode: _jsx("li", {}), dItemRender: (item, index, { iARIA, iChildren }, parent) => {
                                        const { value: itemValue, disabled: itemDisabled, children } = item;
                                        const itemNode = dCustomItem ? dCustomItem(item) : itemValue;
                                        if (children) {
                                            return (_jsxs("ul", Object.assign({ className: `${dPrefix}auto-complete__option-group`, role: "group", "aria-labelledby": getGroupId(itemValue) }, { children: [_jsx("li", Object.assign({ id: getGroupId(itemValue), className: `${dPrefix}auto-complete__option-group-label`, role: "presentation" }, { children: _jsx("div", Object.assign({ className: `${dPrefix}auto-complete__option-content` }, { children: itemNode })) }), itemValue), iChildren] }), itemValue));
                                        }
                                        return (_createElement("li", Object.assign({}, iARIA, { key: itemValue, id: getItemId(itemValue), className: getClassName(`${dPrefix}auto-complete__option`, {
                                                'is-disabled': itemDisabled,
                                            }), style: { paddingLeft: parent.length === 0 ? undefined : 12 + 8 }, title: itemValue, role: "option", "aria-selected": false, "aria-disabled": itemDisabled, onClick: () => {
                                                if (!itemDisabled) {
                                                    changeFocusItem(item);
                                                    changeVisible(false);
                                                    onItemClick === null || onItemClick === void 0 ? void 0 : onItemClick(itemValue, item);
                                                }
                                            } }),
                                            focusVisible && (focusItem === null || focusItem === void 0 ? void 0 : focusItem.value) === itemValue && _jsx("div", { className: `${dPrefix}focus-outline` }),
                                            _jsx("div", Object.assign({ className: `${dPrefix}auto-complete__option-content` }, { children: itemNode }))));
                                    }, dFocusItem: focusItem, dSize: 264, dPadding: 4, dEmptyRender: () => (_jsx("li", Object.assign({ className: `${dPrefix}auto-complete__empty`, style: { paddingLeft: 12 + 8 } }, { children: _jsx("div", Object.assign({ className: `${dPrefix}auto-complete__option-content` }, { children: t('No Data') })) }))), onScrollEnd: onScrollBottom }, { children: ({ vsScrollRef, vsRender, vsOnScroll }) => (_jsx("ul", Object.assign({ ref: vsScrollRef, id: listId, className: `${dPrefix}auto-complete__list`, style: { pointerEvents: dLoading ? 'none' : undefined }, tabIndex: -1, role: "listbox", "aria-activedescendant": isUndefined(focusItem) ? undefined : getItemId(focusItem.value), onScroll: vsOnScroll }, { children: dList.length === 0 ? (_jsx("li", Object.assign({ className: `${dPrefix}auto-complete__empty` }, { children: _jsx("div", Object.assign({ className: `${dPrefix}auto-complete__option-content` }, { children: t('No Data') })) }))) : (vsRender) }))) }))] })));
                    } })), containerEl)] }));
}
export const DAutoComplete = React.forwardRef(AutoComplete);
//# sourceMappingURL=AutoComplete.js.map