var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { isNull } from 'lodash';
import React, { useCallback, useState, useId, useMemo, useRef } from 'react';
import { useEventNotify } from '@react-devui/hooks';
import { CloseOutlined, LoadingOutlined } from '@react-devui/icons';
import { findNested, getClassName, getOriginalSize, getVerticalSidePosition } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useGeneralContext, useDValue, useTranslation } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DComboboxKeyboardSupport } from '../_keyboard-support';
import { DSelectbox } from '../_selectbox';
import { DDropdown } from '../dropdown';
import { useFormControl } from '../form';
import { DTag } from '../tag';
import { DSearchPanel as DTreeSearchPanel } from '../tree/SearchPanel';
import { MultipleTreeNode, SingleTreeNode } from '../tree/node';
import { getText, TREE_NODE_KEY } from '../tree/utils';
import { DList } from './List';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DCascader' });
function Cascader(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dFormControl, dList, dModel, dVisible, dPlaceholder, dSize, dLoading = false, dSearchable = false, dClearable = false, dDisabled = false, dMultiple = false, dOnlyLeafSelectable = true, dCustomItem, dCustomSelected, dCustomSearch, dPopupClassName, dInputProps, dInputRef, onModelChange, onVisibleChange, afterVisibleChange, onSearch, onClear, onFirstFocus } = _a, restProps = __rest(_a, ["dFormControl", "dList", "dModel", "dVisible", "dPlaceholder", "dSize", "dLoading", "dSearchable", "dClearable", "dDisabled", "dMultiple", "dOnlyLeafSelectable", "dCustomItem", "dCustomSelected", "dCustomSearch", "dPopupClassName", "dInputProps", "dInputRef", "onModelChange", "onVisibleChange", "afterVisibleChange", "onSearch", "onClear", "onFirstFocus"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { gSize, gDisabled } = useGeneralContext();
    //#endregion
    const dataRef = useRef({ focusList: new Set() });
    const [t] = useTranslation();
    const onKeyDown$ = useEventNotify();
    const uniqueId = useId();
    const listId = `${dPrefix}cascader-list-${uniqueId}`;
    const getItemId = (val) => `${dPrefix}cascader-item-${val}-${uniqueId}`;
    const renderNodes = useMemo(() => dList.map((item) => dMultiple
        ? new MultipleTreeNode(item, (origin) => origin.value, {
            disabled: item.disabled,
        })
        : new SingleTreeNode(item, (origin) => origin.value, {
            disabled: item.disabled,
        })), [dMultiple, dList]);
    const nodesMap = useMemo(() => {
        const nodes = new Map();
        const reduceArr = (arr) => {
            for (const item of arr) {
                nodes.set(item.id, item);
                if (item.children) {
                    reduceArr(item.children);
                }
            }
        };
        reduceArr(renderNodes);
        return nodes;
    }, [renderNodes]);
    const [searchValue, setSearchValue] = useState('');
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const formControlInject = useFormControl(dFormControl);
    const [_select, changeSelect] = useDValue(dMultiple ? [] : null, dModel, (value) => {
        var _a;
        if (onModelChange) {
            if (dMultiple) {
                onModelChange(value, value.map((v) => { var _a; return (_a = nodesMap.get(v)) === null || _a === void 0 ? void 0 : _a.origin; }));
            }
            else {
                onModelChange(value, isNull(value) ? null : (_a = nodesMap.get(value)) === null || _a === void 0 ? void 0 : _a.origin);
            }
        }
    }, undefined, formControlInject);
    const select = useMemo(() => (dMultiple ? new Set(_select) : _select), [_select, dMultiple]);
    renderNodes.forEach((node) => {
        node.updateStatus(select);
    });
    const size = dSize !== null && dSize !== void 0 ? dSize : gSize;
    const disabled = dDisabled || gDisabled || (dFormControl === null || dFormControl === void 0 ? void 0 : dFormControl.control.disabled);
    const hasSearch = searchValue.length > 0;
    const hasSelected = dMultiple ? select.size > 0 : !isNull(select);
    const [focusVisible, setFocusVisible] = useState(false);
    const filterFn = useCallback((item, searchStr = searchValue) => {
        const defaultFilterFn = (item) => {
            return item.label.includes(searchStr);
        };
        return dCustomSearch && dCustomSearch.filter ? dCustomSearch.filter(searchStr, item) : defaultFilterFn(item);
    }, [dCustomSearch, searchValue]);
    const sortFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.sort;
    const searchList = useMemo(() => {
        if (!hasSearch) {
            return [];
        }
        const searchList = [];
        const reduceNodes = (nodes) => {
            nodes.forEach((node) => {
                if ((!dMultiple && !dOnlyLeafSelectable) || node.isLeaf) {
                    if (filterFn(node.origin)) {
                        searchList.push({
                            label: getText(node),
                            value: node.id,
                            disabled: node.disabled,
                            [TREE_NODE_KEY]: node,
                        });
                    }
                }
                if (node.children) {
                    reduceNodes(node.children);
                }
            });
        };
        reduceNodes(renderNodes);
        if (sortFn) {
            searchList.sort((a, b) => sortFn(a[TREE_NODE_KEY].origin, b[TREE_NODE_KEY].origin));
        }
        return searchList;
    }, [dMultiple, dOnlyLeafSelectable, filterFn, hasSearch, renderNodes, sortFn]);
    const [_noSearchFocusItem, setNoSearchFocusItem] = useState();
    const noSearchFocusItem = (() => {
        if (_noSearchFocusItem) {
            const node = nodesMap.get(_noSearchFocusItem.id);
            if (node && node.enabled) {
                return node;
            }
        }
        if (hasSelected) {
            return findNested(renderNodes, (node) => node.enabled && node.checked);
        }
    })();
    const [_searchFocusItem, setSearchFocusItem] = useState();
    const searchFocusItem = (() => {
        if (_searchFocusItem && findNested(searchList, (item) => item[TREE_NODE_KEY].enabled && item.value === _searchFocusItem.value)) {
            return _searchFocusItem;
        }
        if (hasSearch) {
            return findNested(searchList, (item) => item[TREE_NODE_KEY].enabled);
        }
    })();
    const handleClear = () => {
        onClear === null || onClear === void 0 ? void 0 : onClear();
        if (dMultiple) {
            changeSelect([]);
        }
        else {
            changeSelect(null);
        }
    };
    const [selectedNode, suffixNode, selectedLabel] = (() => {
        let selectedNode = null;
        let suffixNode = null;
        let selectedLabel;
        if (dMultiple) {
            const selectedNodes = _select.map((v) => nodesMap.get(v));
            suffixNode = (_jsx(DDropdown, Object.assign({ dList: selectedNodes.map((node) => {
                    const { value: itemValue, disabled: itemDisabled } = node.origin;
                    const text = dCustomSelected ? dCustomSelected(node.origin) : getText(node);
                    return {
                        id: itemValue,
                        label: text,
                        type: 'item',
                        disabled: itemDisabled,
                        node,
                    };
                }), dCloseOnClick: false, onItemClick: (id, item) => {
                    const checkeds = item.node.changeStatus('UNCHECKED', select);
                    changeSelect(Array.from(checkeds.keys()));
                } }, { children: _jsx(DTag, Object.assign({ className: `${dPrefix}cascader__multiple-count`, dSize: size }, { children: select.size })) })));
            selectedNode = selectedNodes.map((node) => (_jsxs(DTag, Object.assign({ className: `${dPrefix}cascader__multiple-tag`, dSize: size }, { children: [dCustomSelected ? dCustomSelected(node.origin) : node.origin.label, !(node.disabled || disabled) && (_jsx("div", Object.assign({ className: `${dPrefix}cascader__close`, role: "button", "aria-label": t('Close'), onClick: (e) => {
                            e.stopPropagation();
                            const checkeds = node.changeStatus('UNCHECKED', select);
                            changeSelect(Array.from(checkeds.keys()));
                        } }, { children: _jsx(CloseOutlined, {}) })))] }), node.id)));
        }
        else {
            if (!isNull(select)) {
                const node = nodesMap.get(select);
                selectedLabel = getText(node);
                selectedNode = dCustomSelected ? dCustomSelected(node.origin) : selectedLabel;
            }
        }
        return [selectedNode, suffixNode, selectedLabel];
    })();
    return (_jsx(DComboboxKeyboardSupport, Object.assign({ dVisible: visible, dEditable: dSearchable, dNested: !hasSearch, onVisibleChange: changeVisible, onFocusChange: (key) => {
            onKeyDown$.next(key);
        } }, { children: ({ ksOnKeyDown }) => (_jsx(DSelectbox, Object.assign({}, restProps, { ref: ref, onClick: (e) => {
                var _a;
                (_a = restProps.onClick) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                changeVisible((draft) => (dSearchable ? true : !draft));
            }, dClassNamePrefix: "cascader", dFormControl: dFormControl, dVisible: visible, dContent: hasSelected && selectedNode, dContentTitle: selectedLabel, dPlaceholder: dPlaceholder, dSuffix: suffixNode, dSize: size, dLoading: dLoading, dSearchable: dSearchable, dClearable: dClearable, dDisabled: disabled, dInputProps: Object.assign(Object.assign({}, dInputProps), { value: searchValue, 'aria-controls': listId, onBlur: (e) => {
                    var _a;
                    (_a = dInputProps === null || dInputProps === void 0 ? void 0 : dInputProps.onBlur) === null || _a === void 0 ? void 0 : _a.call(dInputProps, e);
                    changeVisible(false);
                }, onKeyDown: (e) => {
                    var _a;
                    (_a = dInputProps === null || dInputProps === void 0 ? void 0 : dInputProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(dInputProps, e);
                    ksOnKeyDown(e);
                    if ((e.code === 'Enter' || (!dSearchable && e.code === 'Space')) && visible) {
                        onKeyDown$.next('click');
                    }
                }, onChange: (e) => {
                    var _a;
                    (_a = dInputProps === null || dInputProps === void 0 ? void 0 : dInputProps.onChange) === null || _a === void 0 ? void 0 : _a.call(dInputProps, e);
                    const val = e.currentTarget.value;
                    if (dSearchable) {
                        setSearchValue(val);
                        onSearch === null || onSearch === void 0 ? void 0 : onSearch(val);
                    }
                } }), dInputRef: dInputRef, dUpdatePosition: (boxEl, popupEl) => {
                const width = boxEl.getBoundingClientRect().width;
                const { height } = getOriginalSize(popupEl);
                const { top, left, transformOrigin } = getVerticalSidePosition(boxEl, { width, height }, 'bottom-left', 8);
                return {
                    position: {
                        top,
                        left,
                        maxWidth: window.innerWidth - left - 20,
                    },
                    transformOrigin,
                };
            }, afterVisibleChange: afterVisibleChange, onFocusVisibleChange: setFocusVisible, onClear: handleClear }, { children: ({ sPopupRef, sStyle, sOnMouseDown, sOnMouseUp }) => (_jsxs("div", Object.assign({ ref: sPopupRef, className: getClassName(dPopupClassName, `${dPrefix}cascader__popup`), style: sStyle, onMouseDown: sOnMouseDown, onMouseUp: sOnMouseUp }, { children: [dLoading && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}cascader__loading`, {
                            [`${dPrefix}cascader__loading--empty`]: (hasSearch ? searchList : renderNodes).length === 0,
                        }) }, { children: _jsx(LoadingOutlined, { dSize: (hasSearch ? searchList : renderNodes).length === 0 ? 18 : 24, dSpin: true }) }))), hasSearch ? (_jsx(DTreeSearchPanel, { id: listId, style: { pointerEvents: dLoading ? 'none' : undefined }, dGetItemId: getItemId, dList: searchList, dFocusItem: searchFocusItem, dCustomItem: dCustomItem, dMultiple: dMultiple, dOnlyLeafSelectable: dOnlyLeafSelectable, dFocusVisible: focusVisible, onFocusChange: (item) => {
                            if (!dataRef.current.focusList.has(item.value)) {
                                dataRef.current.focusList.add(item.value);
                                onFirstFocus === null || onFirstFocus === void 0 ? void 0 : onFirstFocus(item.value, item[TREE_NODE_KEY].origin);
                            }
                            setSearchFocusItem(item);
                        }, onClickItem: (item) => {
                            if (dMultiple) {
                                const checkeds = item[TREE_NODE_KEY].changeStatus(item[TREE_NODE_KEY].checked ? 'UNCHECKED' : 'CHECKED', select);
                                changeSelect(Array.from(checkeds.keys()));
                            }
                            else {
                                changeSelect(item[TREE_NODE_KEY].id);
                                changeVisible(false);
                            }
                        }, "onKeyDown$": onKeyDown$ })) : (_jsx(DList, { id: listId, style: { pointerEvents: dLoading ? 'none' : undefined }, dGetItemId: getItemId, dList: renderNodes, dFocusItem: noSearchFocusItem, dCustomItem: dCustomItem, dMultiple: dMultiple, dFocusVisible: focusVisible, dRoot: true, onFocusChange: (node) => {
                            if (!dataRef.current.focusList.has(node.id)) {
                                dataRef.current.focusList.add(node.id);
                                onFirstFocus === null || onFirstFocus === void 0 ? void 0 : onFirstFocus(node.id, node.origin);
                            }
                            setNoSearchFocusItem(node);
                        }, onClickItem: (node) => {
                            if (dMultiple) {
                                const checkeds = node.changeStatus(node.checked ? 'UNCHECKED' : 'CHECKED', select);
                                changeSelect(Array.from(checkeds.keys()));
                            }
                            else {
                                if (!dOnlyLeafSelectable || node.isLeaf) {
                                    changeSelect(node.id);
                                }
                                if (node.isLeaf) {
                                    changeVisible(false);
                                }
                            }
                        }, "onKeyDown$": onKeyDown$ }))] }))) }))) })));
}
export const DCascader = React.forwardRef(Cascader);
//# sourceMappingURL=Cascader.js.map