import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useEffect, useImperativeHandle, useRef, useState } from 'react';
import { useAsync, useEventCallback } from '@react-devui/hooks';
import { DoubleLeftOutlined, DoubleRightOutlined, LeftOutlined, RightOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import { usePrefixConfig, useTranslation } from '../../hooks';
import { dayjs } from '../dayjs';
function Panel(props, ref) {
    const { dDate, dAnotherDate, dConfigDate, onDateChange } = props;
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    const dataRef = useRef({});
    const asyncCapture = useAsync();
    const [t, lang] = useTranslation();
    const activeDate = dDate ? dayjs(dDate) : dayjs().set('hour', 0).set('minute', 0).set('second', 0);
    const anotherDate = dAnotherDate ? dayjs(dAnotherDate) : null;
    const [hoverDate, setHoverDate] = useState(null);
    const [showDate, setShowDate] = useState(activeDate);
    const updateView = useEventCallback((t) => {
        setShowDate(dayjs(t));
    });
    const handleButtonDown = (unit, value) => {
        const loop = () => {
            setShowDate((d) => d.set(unit, d.get(unit) + value));
            dataRef.current.clearLoop = asyncCapture.setTimeout(() => loop(), 50);
        };
        dataRef.current.clearTid = asyncCapture.setTimeout(() => loop(), 400);
    };
    const handleButtonMouseUp = () => {
        var _a, _b, _c, _d;
        (_b = (_a = dataRef.current).clearLoop) === null || _b === void 0 ? void 0 : _b.call(_a);
        (_d = (_c = dataRef.current).clearTid) === null || _d === void 0 ? void 0 : _d.call(_c);
    };
    const getButtonProps = (unit, value) => ({
        onMouseDown: (e) => {
            if (e.button === 0) {
                handleButtonDown(unit, value);
            }
        },
        onTouchStart: () => {
            handleButtonDown(unit, value);
        },
        onTouchEnd: () => {
            handleButtonMouseUp();
        },
        onClick: () => {
            updateView(showDate.set(unit, showDate.get(unit) + value).toDate());
        },
    });
    const weekList = (() => {
        const firstDay = showDate.set('date', 1);
        const month = [];
        let week = [];
        for (let num = 0, addDay = -firstDay.day(); num < 7 * 6; num++, addDay++) {
            week.push(firstDay.add(addDay, 'day'));
            if (week.length === 7) {
                month.push(week);
                week = [];
            }
        }
        return month;
    })();
    useEffect(() => {
        const [asyncGroup, asyncId] = asyncCapture.createGroup();
        asyncGroup.fromEvent(window, 'mouseup').subscribe({
            next: () => {
                handleButtonMouseUp();
            },
        });
        return () => {
            asyncCapture.deleteGroup(asyncId);
        };
    }, [asyncCapture]);
    useImperativeHandle(ref, () => ({
        updateView,
    }), [updateView]);
    return (_jsxs("div", Object.assign({ className: `${dPrefix}date-picker__panel` }, { children: [_jsxs("div", Object.assign({ className: `${dPrefix}date-picker__header` }, { children: [_jsx("button", Object.assign({}, getButtonProps('year', -1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Previous year') }, { children: _jsx(DoubleLeftOutlined, {}) })), _jsx("button", Object.assign({}, getButtonProps('month', -1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Previous month') }, { children: _jsx(LeftOutlined, {}) })), _jsx("span", Object.assign({ className: `${dPrefix}date-picker__header-content` }, { children: showDate.format(lang === 'zh-Hant' ? 'YYYY年 M月' : 'MMM YYYY') })), _jsx("button", Object.assign({}, getButtonProps('month', 1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Next month') }, { children: _jsx(RightOutlined, {}) })), _jsx("button", Object.assign({}, getButtonProps('year', 1), { className: `${dPrefix}date-picker__header-button`, title: t('DatePicker', 'Next year') }, { children: _jsx(DoubleRightOutlined, {}) }))] })), _jsxs("table", Object.assign({ className: `${dPrefix}date-picker__content` }, { children: [_jsx("thead", { children: _jsx("tr", { children: dayjs.weekdaysMin().map((day) => (_jsx("th", { children: day }, day))) }) }), _jsx("tbody", { children: weekList.map((week, index1) => (_jsx("tr", { children: week.map((d, index2) => {
                                var _a;
                                const { disabled } = (_a = dConfigDate === null || dConfigDate === void 0 ? void 0 : dConfigDate(d.toDate())) !== null && _a !== void 0 ? _a : {};
                                const isActive = dDate && d.isSame(activeDate, 'date');
                                const isAnother = anotherDate && d.isSame(anotherDate, 'date');
                                const isHover = !isActive && !isAnother && anotherDate && d.isSame(hoverDate, 'date');
                                return (_jsx("td", Object.assign({ style: { pointerEvents: disabled ? 'none' : undefined }, onClick: () => {
                                        updateView(d.toDate());
                                        onDateChange === null || onDateChange === void 0 ? void 0 : onDateChange(d.toDate());
                                    }, onMouseEnter: () => {
                                        var _a, _b;
                                        (_b = (_a = dataRef.current).clearHoverTid) === null || _b === void 0 ? void 0 : _b.call(_a);
                                        setHoverDate(d);
                                    }, onMouseLeave: () => {
                                        var _a, _b;
                                        (_b = (_a = dataRef.current).clearHoverTid) === null || _b === void 0 ? void 0 : _b.call(_a);
                                        dataRef.current.clearHoverTid = asyncCapture.setTimeout(() => setHoverDate(null), 50);
                                    } }, { children: _jsx("div", Object.assign({ className: getClassName(`${dPrefix}date-picker__cell`, {
                                            [`${dPrefix}date-picker__cell--out-month`]: d.get('month') !== showDate.get('month'),
                                            [`${dPrefix}date-picker__cell--today`]: !isActive && !isAnother && d.isSame(dayjs(), 'date'),
                                            'is-active': isActive,
                                            'is-another': !isActive && isAnother,
                                            'is-hover': isHover,
                                            'is-between': !isActive && !isAnother && !isHover && dDate && anotherDate && d.isBetween(activeDate, anotherDate, 'date'),
                                            'is-between-hover': !isActive && hoverDate && anotherDate && d.isBetween(hoverDate, anotherDate, 'date'),
                                            'is-disabled': disabled,
                                        }) }, { children: d.get('date') })) }), index2));
                            }) }, index1))) })] }))] })));
}
export const DPanel = React.forwardRef(Panel);
//# sourceMappingURL=Panel.js.map