var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isBoolean, isFunction, isNull, isNumber, isString, isUndefined } from 'lodash';
import { useContext, useEffect, useId, useRef } from 'react';
import { CheckCircleFilled, CloseCircleFilled, ExclamationCircleFilled, LoadingOutlined, QuestionCircleOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useTranslation, useGridConfig, useContextRequired } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DTooltip } from '../tooltip';
import { DError } from './Error';
import { DFormContext } from './Form';
import { DFormGroupContext } from './FormGroup';
import { Validators } from './form-control';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DForm.Item' });
export function DFormItem(props) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { children, dFormControls = {}, dLabel, dLabelWidth, dLabelExtra, dShowRequired, dSpan, dResponsiveProps } = _a, restProps = __rest(_a, ["children", "dFormControls", "dLabel", "dLabelWidth", "dLabelExtra", "dShowRequired", "dSpan", "dResponsiveProps"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dColNum } = useGridConfig();
    const { gBreakpointMatchs, gLabelWidth, gLabelColon, gRequiredType, gLayout, gInlineSpan, gFeedbackIcon } = useContextRequired(DFormContext);
    const formGroup = useContext(DFormGroupContext);
    //#endregion
    //#region Ref
    const labelRef = useRef(null);
    const contentRef = useRef(null);
    //#endregion
    const [t] = useTranslation();
    const uniqueId = useId();
    const getErrorId = (formControlName) => `${dPrefix}form-item-error-${formControlName}-${uniqueId}`;
    const formControls = (() => {
        const obj = {};
        Object.keys(dFormControls).forEach((formControlName) => {
            const formControl = formGroup.get(formControlName);
            if (isNull(formControl)) {
                throw new Error(`Cant find '${formControlName}', please check if name exists!`);
            }
            obj[formControlName] = {
                control: formControl,
                wrapperAttrs: {},
                inputAttrs: {
                    'aria-invalid': formControl.enabled && formControl.dirty && formControl.invalid,
                },
            };
        });
        return obj;
    })();
    const { span, labelWidth } = (() => {
        const props = {
            span: dSpan !== null && dSpan !== void 0 ? dSpan : (gLayout === 'inline' ? gInlineSpan : dColNum),
            labelWidth: dLabelWidth !== null && dLabelWidth !== void 0 ? dLabelWidth : gLabelWidth,
        };
        if (dResponsiveProps) {
            const mergeProps = (point, targetKey, sourceKey) => {
                const value = dResponsiveProps[point][sourceKey];
                if (!isUndefined(value)) {
                    props[targetKey] = value;
                }
            };
            for (const breakpoint of gBreakpointMatchs) {
                if (breakpoint in dResponsiveProps) {
                    mergeProps(breakpoint, 'span', 'dSpan');
                    mergeProps(breakpoint, 'labelWidth', 'dLabelWidth');
                    break;
                }
            }
        }
        return props;
    })();
    const prevErrors = useRef([]);
    const [errorNodes, formItemStatus] = (() => {
        const errors = [];
        let formItemStatus;
        Object.entries(dFormControls).forEach(([formControlName, errorInfo]) => {
            const { control } = formControls[formControlName];
            if (control.enabled && control.dirty) {
                let status = 'success';
                if (control.status === 'PENDING') {
                    status = formItemStatus = 'pending';
                }
                else if (control.status === 'INVALID') {
                    status = 'warning';
                    if (formItemStatus !== 'pending' && formItemStatus !== 'error') {
                        formItemStatus = 'warning';
                    }
                }
                else if (control.status === 'VALID') {
                    status = 'success';
                    if (formItemStatus === undefined) {
                        formItemStatus = 'success';
                    }
                }
                let hasError = false;
                if (control.invalid && control.errors) {
                    if (isString(errorInfo)) {
                        errors.push({ key: formControlName, formControlName, message: errorInfo, status: 'error' });
                        hasError = true;
                    }
                    else if (Object.keys(errorInfo).length === 2 && 'message' in errorInfo && 'status' in errorInfo) {
                        errors.push(Object.assign({ key: formControlName, formControlName }, errorInfo));
                        if (errorInfo.status === 'error') {
                            hasError = true;
                        }
                    }
                    else if (control.errors) {
                        for (const key of Object.keys(control.errors)) {
                            if (key in errorInfo) {
                                if (isString(errorInfo[key])) {
                                    errors.push({ key: `${formControlName}-${key}`, formControlName, message: errorInfo[key], status: 'error' });
                                    hasError = true;
                                }
                                else {
                                    errors.push(Object.assign({ key: `${formControlName}-${key}`, formControlName }, errorInfo[key]));
                                    if (errorInfo[key].status === 'error') {
                                        hasError = true;
                                    }
                                }
                            }
                        }
                    }
                }
                if (hasError) {
                    if (status === 'warning') {
                        status = 'error';
                    }
                    if (formItemStatus === 'warning') {
                        formItemStatus = 'error';
                    }
                }
                if (status !== 'success') {
                    formControls[formControlName].wrapperAttrs[`data-form-invalid-${status}`] = true;
                }
            }
        });
        prevErrors.current.forEach((error, inedx) => {
            if (errors.findIndex((item) => item.key === error.key) === -1) {
                errors.splice(inedx, 0, Object.assign(Object.assign({}, error), { hidden: true }));
            }
        });
        prevErrors.current = errors;
        const errorNames = new Set(errors.map((item) => item.formControlName));
        const errorNodes = [];
        errorNames.forEach((formControlName) => {
            const id = getErrorId(formControlName);
            formControls[formControlName].inputAttrs['aria-describedby'] = id;
            errorNodes.push(_jsx("div", Object.assign({ id: id }, { children: errors
                    .filter((item) => item.formControlName === formControlName)
                    .map((error) => (_jsx(DError, { dVisible: !error.hidden, dMessage: error.message, dStatus: error.status, onHidden: () => {
                        prevErrors.current = prevErrors.current.filter((item) => item.key !== error.key);
                    } }, error.key))) }), formControlName));
        });
        return [errorNodes, formItemStatus];
    })();
    const required = (() => {
        if (isBoolean(dShowRequired)) {
            return dShowRequired;
        }
        for (const { control } of Object.values(formControls)) {
            if (control.hasValidator(Validators.required)) {
                return true;
            }
        }
        return false;
    })();
    const feedbackIcon = (() => {
        var _a;
        if (isUndefined(formItemStatus)) {
            return null;
        }
        else {
            const statusIcons = {
                pending: _jsx(LoadingOutlined, { dSpin: true }),
                error: _jsx(CloseCircleFilled, {}),
                warning: _jsx(ExclamationCircleFilled, {}),
                success: _jsx(CheckCircleFilled, {}),
            };
            if (isBoolean(gFeedbackIcon)) {
                return statusIcons[formItemStatus];
            }
            else {
                return (_a = gFeedbackIcon[formItemStatus]) !== null && _a !== void 0 ? _a : statusIcons[formItemStatus];
            }
        }
    })();
    const extraNode = (() => {
        if (dLabelExtra) {
            return dLabelExtra.map((extra, index) => {
                var _a;
                return isString(extra) ? (_jsx("div", { children: extra }, index)) : (_jsx(DTooltip, Object.assign({ dTitle: extra.title }, { children: (_a = extra.icon) !== null && _a !== void 0 ? _a : _jsx(QuestionCircleOutlined, { style: { cursor: 'pointer' }, dSize: "1.1em" }) }), index));
            });
        }
    })();
    const contentWidth = gLayout === 'vertical' ? '100%' : `calc(100% - ${isNumber(labelWidth) ? labelWidth + 'px' : labelWidth})`;
    useEffect(() => {
        if (labelRef.current && contentRef.current) {
            const el = contentRef.current.querySelector(`[data-form-label-for="true"]`);
            if (el) {
                labelRef.current.setAttribute('for', el.id);
            }
        }
    });
    return (_jsxs("div", Object.assign({}, restProps, { className: getClassName(restProps.className, `${dPrefix}form__item`, {
            [`${dPrefix}form__item--vertical`]: gLayout === 'vertical',
        }), style: Object.assign(Object.assign({}, restProps.style), { flexGrow: span === true ? 1 : undefined, flexShrink: span === true ? undefined : 0, width: span === true ? undefined : isNumber(span) ? `calc((100% / ${dColNum}) * ${span})` : span }) }, { children: [_jsxs("div", Object.assign({ className: `${dPrefix}form__item-container` }, { children: [labelWidth !== 0 &&
                        (dLabel ? (_jsxs("label", Object.assign({ ref: labelRef, className: getClassName(`${dPrefix}form__item-label`, {
                                [`${dPrefix}form__item-label--required`]: gRequiredType === 'required' && required,
                                [`${dPrefix}form__item-label--colon`]: gLabelColon,
                            }), style: { width: gLayout === 'vertical' ? undefined : labelWidth } }, { children: [dLabel, (extraNode || (gRequiredType === 'optional' && !required)) && (_jsxs("div", Object.assign({ className: `${dPrefix}form__item-label-extra` }, { children: [extraNode, gRequiredType === 'optional' && !required && _jsx("div", { children: t('Form', 'Optional') })] })))] }))) : (_jsx("div", { style: { width: labelWidth } }))), _jsxs("div", Object.assign({ ref: contentRef, className: `${dPrefix}form__item-content`, style: { width: contentWidth } }, { children: [formItemStatus === 'pending' && (_jsxs(_Fragment, { children: [_jsx("div", { className: getClassName(`${dPrefix}form__pending`, `${dPrefix}form__pending--1`) }), _jsx("div", { className: getClassName(`${dPrefix}form__pending`, `${dPrefix}form__pending--2`) }), _jsx("div", { className: getClassName(`${dPrefix}form__pending`, `${dPrefix}form__pending--3`) }), _jsx("div", { className: getClassName(`${dPrefix}form__pending`, `${dPrefix}form__pending--4`) })] })), isFunction(children) ? children(formControls) : children] })), gFeedbackIcon && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}form__feedback-icon`, {
                            [`is-${formItemStatus}`]: formItemStatus,
                        }) }, { children: feedbackIcon })))] })), _jsx("div", Object.assign({ className: `${dPrefix}form__error-container`, style: { width: contentWidth } }, { children: errorNodes }))] })));
}
//# sourceMappingURL=FormItem.js.map