var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createElement as _createElement } from "react";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { isNull, isUndefined } from 'lodash';
import React, { useState, useId, useCallback, useMemo, useRef } from 'react';
import { useEventCallback } from '@react-devui/hooks';
import { CloseOutlined, LoadingOutlined, PlusOutlined } from '@react-devui/icons';
import { findNested, getClassName, getOriginalSize, getVerticalSidePosition } from '@react-devui/utils';
import { usePrefixConfig, useComponentConfig, useTranslation, useGeneralContext, useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DComboboxKeyboardSupport } from '../_keyboard-support';
import { DSelectbox } from '../_selectbox';
import { DCheckbox } from '../checkbox';
import { DDropdown } from '../dropdown';
import { useFormControl } from '../form';
import { DTag } from '../tag';
import { DVirtualScroll } from '../virtual-scroll';
const IS_CREATE = Symbol();
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DSelect' });
function Select(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dFormControl, dList, dModel, dVisible, dPlaceholder, dSize, dLoading = false, dSearchable = false, dClearable = false, dDisabled = false, dMultiple = false, dCustomItem, dCustomSelected, dCustomSearch, dCreateItem, dPopupClassName, dInputProps, dInputRef, onModelChange, onVisibleChange, afterVisibleChange, onClear, onSearch, onCreateItem, onScrollBottom } = _a, restProps = __rest(_a, ["dFormControl", "dList", "dModel", "dVisible", "dPlaceholder", "dSize", "dLoading", "dSearchable", "dClearable", "dDisabled", "dMultiple", "dCustomItem", "dCustomSelected", "dCustomSearch", "dCreateItem", "dPopupClassName", "dInputProps", "dInputRef", "onModelChange", "onVisibleChange", "afterVisibleChange", "onClear", "onSearch", "onCreateItem", "onScrollBottom"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { gSize, gDisabled } = useGeneralContext();
    //#endregion
    //#region Ref
    const dVSRef = useRef(null);
    //#endregion
    const [t] = useTranslation();
    const uniqueId = useId();
    const listId = `${dPrefix}select-list-${uniqueId}`;
    const getGroupId = (val) => `${dPrefix}select-group-${val}-${uniqueId}`;
    const getItemId = (val) => `${dPrefix}select-item-${val}-${uniqueId}`;
    const itemsMap = useMemo(() => {
        const items = new Map();
        const reduceArr = (arr) => {
            for (const item of arr) {
                items.set(item.value, item);
                if (item.children) {
                    reduceArr(item.children);
                }
            }
        };
        reduceArr(dList);
        return items;
    }, [dList]);
    const [searchValue, setSearchValue] = useState('');
    const canSelectItem = useCallback((item) => !item.disabled && !item.children, []);
    const [focusVisible, setFocusVisible] = useState(false);
    const [visible, changeVisible] = useDValue(false, dVisible, onVisibleChange);
    const formControlInject = useFormControl(dFormControl);
    const [_select, changeSelect] = useDValue(dMultiple ? [] : null, dModel, (value) => {
        if (onModelChange) {
            if (dMultiple) {
                onModelChange(value, value.map((v) => itemsMap.get(v)));
            }
            else {
                onModelChange(value, isNull(value) ? null : itemsMap.get(value));
            }
        }
    }, undefined, formControlInject);
    const select = useMemo(() => (dMultiple ? new Set(_select) : _select), [_select, dMultiple]);
    const size = dSize !== null && dSize !== void 0 ? dSize : gSize;
    const disabled = dDisabled || gDisabled || (dFormControl === null || dFormControl === void 0 ? void 0 : dFormControl.control.disabled);
    const hasSearch = searchValue.length > 0;
    const hasSelected = dMultiple ? select.size > 0 : !isNull(select);
    const _filterFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.filter;
    const filterFn = useCallback((item) => {
        const defaultFilterFn = (item) => {
            return item.label.includes(searchValue);
        };
        return _filterFn ? _filterFn(searchValue, item) : defaultFilterFn(item);
    }, [_filterFn, searchValue]);
    const sortFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.sort;
    const searchList = useMemo(() => {
        if (!hasSearch) {
            return [];
        }
        let createItem = dCreateItem === null || dCreateItem === void 0 ? void 0 : dCreateItem(searchValue);
        if (createItem) {
            createItem = Object.assign(Object.assign({}, createItem), { [IS_CREATE]: true });
        }
        let searchList = [];
        dList.forEach((item) => {
            if (!item.children) {
                if (createItem && item.value === createItem.value) {
                    createItem = undefined;
                }
                if (filterFn(item)) {
                    searchList.push(item);
                }
            }
            else {
                const groupList = [];
                item.children.forEach((groupItem) => {
                    if (createItem && groupItem.value === createItem.value) {
                        createItem = undefined;
                    }
                    if (filterFn(groupItem)) {
                        groupList.push(groupItem);
                    }
                });
                searchList = searchList.concat(groupList);
            }
        });
        if (sortFn) {
            searchList.sort(sortFn);
        }
        if (createItem) {
            searchList.unshift(createItem);
        }
        return searchList;
    }, [dCreateItem, dList, filterFn, hasSearch, searchValue, sortFn]);
    const list = hasSearch ? searchList : dList;
    const [_noSearchFocusItem, setNoSearchFocusItem] = useState();
    const noSearchFocusItem = (() => {
        let focusItem;
        if (_noSearchFocusItem) {
            focusItem = itemsMap.get(_noSearchFocusItem.value);
            if (focusItem && canSelectItem(focusItem)) {
                return focusItem;
            }
        }
        if (hasSelected) {
            focusItem = findNested(dList, (item) => canSelectItem(item) && dMultiple ? select.has(item.value) : select === item.value);
        }
        if (isUndefined(focusItem)) {
            focusItem = findNested(dList, (item) => canSelectItem(item));
        }
        return focusItem;
    })();
    const [_searchFocusItem, setSearchFocusItem] = useState();
    const searchFocusItem = (() => {
        if (_searchFocusItem && findNested(searchList, (item) => canSelectItem(item) && item.value === _searchFocusItem.value)) {
            return _searchFocusItem;
        }
        if (hasSearch) {
            return findNested(searchList, (item) => canSelectItem(item));
        }
    })();
    const focusItem = hasSearch ? searchFocusItem : noSearchFocusItem;
    const changeFocusItem = (item) => {
        if (!isUndefined(item)) {
            hasSearch ? setSearchFocusItem(item) : setNoSearchFocusItem(item);
        }
    };
    const createItem = (item) => {
        if (!isUndefined(item)) {
            const newItem = Object.assign({}, item);
            delete newItem[IS_CREATE];
            onCreateItem === null || onCreateItem === void 0 ? void 0 : onCreateItem(newItem);
        }
    };
    const changeSelectByClick = useEventCallback((val) => {
        if (dMultiple) {
            changeSelect((draft) => {
                const index = draft.findIndex((v) => v === val);
                if (index !== -1) {
                    draft.splice(index, 1);
                }
                else {
                    draft.push(val);
                }
            });
        }
        else {
            changeSelect(val);
            changeVisible(false);
        }
    });
    const [selectedNode, suffixNode, selectedLabel] = (() => {
        let selectedNode = null;
        let suffixNode = null;
        let selectedLabel;
        if (dMultiple) {
            const selectedItems = _select.map((v) => itemsMap.get(v));
            suffixNode = (_jsx(DDropdown, Object.assign({ dList: selectedItems.map((item) => {
                    const { label: itemLabel, value: itemValue, disabled: itemDisabled } = item;
                    const text = dCustomSelected ? dCustomSelected(item) : itemLabel;
                    return {
                        id: itemValue,
                        label: text,
                        type: 'item',
                        disabled: itemDisabled,
                    };
                }), dCloseOnClick: false, onItemClick: (id) => {
                    changeSelectByClick(id);
                } }, { children: _jsx(DTag, Object.assign({ className: `${dPrefix}select__multiple-count`, dSize: size }, { children: select.size })) })));
            selectedNode = selectedItems.map((item) => (_jsxs(DTag, Object.assign({ className: `${dPrefix}select__multiple-tag`, dSize: size }, { children: [dCustomSelected ? dCustomSelected(item) : item.label, !(item.disabled || disabled) && (_jsx("div", Object.assign({ className: `${dPrefix}select__close`, role: "button", "aria-label": t('Close'), onClick: (e) => {
                            e.stopPropagation();
                            changeSelectByClick(item.value);
                        } }, { children: _jsx(CloseOutlined, {}) })))] }), item.value)));
        }
        else {
            if (!isNull(select)) {
                const item = itemsMap.get(select);
                selectedLabel = item.label;
                selectedNode = dCustomSelected ? dCustomSelected(item) : selectedLabel;
            }
        }
        return [selectedNode, suffixNode, selectedLabel];
    })();
    const vsPerformance = useMemo(() => ({
        dList: list,
        dItemSize: 32,
        dItemNested: (item) => ({
            list: item.children,
            emptySize: 32,
            asItem: false,
        }),
        dItemKey: (item) => item.value,
        dFocusable: canSelectItem,
    }), [canSelectItem, list]);
    return (_jsx(DComboboxKeyboardSupport, Object.assign({ dVisible: visible, dEditable: dSearchable, onVisibleChange: changeVisible, onFocusChange: (key) => {
            var _a, _b, _c, _d;
            switch (key) {
                case 'next':
                    changeFocusItem((_a = dVSRef.current) === null || _a === void 0 ? void 0 : _a.scrollByStep(1));
                    break;
                case 'prev':
                    changeFocusItem((_b = dVSRef.current) === null || _b === void 0 ? void 0 : _b.scrollByStep(-1));
                    break;
                case 'first':
                    changeFocusItem((_c = dVSRef.current) === null || _c === void 0 ? void 0 : _c.scrollToStart());
                    break;
                case 'last':
                    changeFocusItem((_d = dVSRef.current) === null || _d === void 0 ? void 0 : _d.scrollToEnd());
                    break;
                default:
                    break;
            }
        } }, { children: ({ ksOnKeyDown }) => (_jsx(DSelectbox, Object.assign({}, restProps, { ref: ref, onClick: (e) => {
                var _a;
                (_a = restProps.onClick) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                changeVisible((draft) => (dSearchable ? true : !draft));
            }, dClassNamePrefix: "select", dFormControl: dFormControl, dVisible: visible, dContent: hasSelected && selectedNode, dContentTitle: selectedLabel, dPlaceholder: dPlaceholder, dSuffix: suffixNode, dSize: size, dLoading: dLoading, dSearchable: dSearchable, dClearable: dClearable, dDisabled: disabled, dInputProps: Object.assign(Object.assign({}, dInputProps), { value: searchValue, 'aria-controls': listId, onBlur: (e) => {
                    var _a;
                    (_a = dInputProps === null || dInputProps === void 0 ? void 0 : dInputProps.onBlur) === null || _a === void 0 ? void 0 : _a.call(dInputProps, e);
                    changeVisible(false);
                }, onKeyDown: (e) => {
                    var _a;
                    (_a = dInputProps === null || dInputProps === void 0 ? void 0 : dInputProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(dInputProps, e);
                    ksOnKeyDown(e);
                    if ((e.code === 'Enter' || (!dSearchable && e.code === 'Space')) && visible && focusItem) {
                        e.preventDefault();
                        if (focusItem[IS_CREATE]) {
                            createItem(focusItem);
                        }
                        changeSelectByClick(focusItem.value);
                    }
                }, onChange: (e) => {
                    var _a;
                    (_a = dInputProps === null || dInputProps === void 0 ? void 0 : dInputProps.onChange) === null || _a === void 0 ? void 0 : _a.call(dInputProps, e);
                    const val = e.currentTarget.value;
                    if (dSearchable) {
                        setSearchValue(val);
                        onSearch === null || onSearch === void 0 ? void 0 : onSearch(val);
                    }
                } }), dInputRef: dInputRef, dUpdatePosition: (boxEl, popupEl) => {
                const width = boxEl.getBoundingClientRect().width;
                const { height } = getOriginalSize(popupEl);
                const { top, left, transformOrigin } = getVerticalSidePosition(boxEl, { width, height }, 'bottom-left', 8);
                return {
                    position: {
                        top,
                        left,
                        width,
                        maxWidth: window.innerWidth - left - 20,
                    },
                    transformOrigin,
                };
            }, afterVisibleChange: afterVisibleChange, onFocusVisibleChange: setFocusVisible, onClear: () => {
                onClear === null || onClear === void 0 ? void 0 : onClear();
                if (dMultiple) {
                    changeSelect([]);
                }
                else {
                    changeSelect(null);
                }
            } }, { children: ({ sPopupRef, sStyle, sOnMouseDown, sOnMouseUp }) => (_jsxs("div", Object.assign({ ref: sPopupRef, className: getClassName(dPopupClassName, `${dPrefix}select__popup`), style: sStyle, onMouseDown: sOnMouseDown, onMouseUp: sOnMouseUp }, { children: [dLoading && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}select__loading`, {
                            [`${dPrefix}select__loading--empty`]: list.length === 0,
                        }) }, { children: _jsx(LoadingOutlined, { dSize: list.length === 0 ? 18 : 24, dSpin: true }) }))), _jsx(DVirtualScroll, Object.assign({}, vsPerformance, { ref: dVSRef, dFillNode: _jsx("li", {}), dItemRender: (item, index, { iARIA, iChildren }, parent) => {
                            const { label: itemLabel, value: itemValue, disabled: itemDisabled, children } = item;
                            const node = dCustomItem ? dCustomItem(item) : itemLabel;
                            if (children) {
                                return (_jsxs("ul", Object.assign({ className: `${dPrefix}select__option-group`, role: "group", "aria-labelledby": getGroupId(itemValue) }, { children: [_jsx("li", Object.assign({ id: getGroupId(itemValue), className: `${dPrefix}select__option-group-label`, role: "presentation" }, { children: _jsx("div", Object.assign({ className: `${dPrefix}select__option-content` }, { children: node })) }), itemValue), iChildren] }), itemValue));
                            }
                            let isSelected = false;
                            if (dMultiple) {
                                isSelected = select.has(itemValue);
                            }
                            else {
                                isSelected = select === itemValue;
                            }
                            return (_createElement("li", Object.assign({}, iARIA, { key: itemValue, id: getItemId(itemValue), className: getClassName(`${dPrefix}select__option`, {
                                    'is-selected': !dMultiple && isSelected,
                                    'is-disabled': itemDisabled,
                                }), style: { paddingLeft: parent.length === 0 ? undefined : 12 + 8 }, title: (item[IS_CREATE] ? t('Create') + ' ' : '') + itemLabel, role: "option", "aria-selected": isSelected, "aria-disabled": itemDisabled, onClick: () => {
                                    if (!itemDisabled) {
                                        if (item[IS_CREATE]) {
                                            createItem(item);
                                        }
                                        changeFocusItem(item);
                                        changeSelectByClick(itemValue);
                                    }
                                } }),
                                focusVisible && (focusItem === null || focusItem === void 0 ? void 0 : focusItem.value) === itemValue && _jsx("div", { className: `${dPrefix}focus-outline` }),
                                item[IS_CREATE] ? (_jsx(PlusOutlined, { dTheme: "primary" })) : dMultiple ? (_jsx(DCheckbox, { dDisabled: itemDisabled, dModel: isSelected })) : null,
                                _jsx("div", Object.assign({ className: `${dPrefix}select__option-content` }, { children: node }))));
                        }, dFocusItem: focusItem, dSize: 264, dPadding: 4, dEmptyRender: () => (_jsx("li", Object.assign({ className: `${dPrefix}select__empty`, style: { paddingLeft: 12 + 8 } }, { children: _jsx("div", Object.assign({ className: `${dPrefix}select__option-content` }, { children: t('No Data') })) }))), onScrollEnd: onScrollBottom }, { children: ({ vsScrollRef, vsRender, vsOnScroll }) => (_jsx("ul", Object.assign({ ref: vsScrollRef, id: listId, className: `${dPrefix}select__list`, style: { pointerEvents: dLoading ? 'none' : undefined }, tabIndex: -1, role: "listbox", "aria-multiselectable": dMultiple, "aria-activedescendant": isUndefined(focusItem) ? undefined : getItemId(focusItem.value), onScroll: vsOnScroll }, { children: list.length === 0 ? (_jsx("li", Object.assign({ className: `${dPrefix}select__empty` }, { children: _jsx("div", Object.assign({ className: `${dPrefix}select__option-content` }, { children: t('No Data') })) }))) : (vsRender) }))) }))] }))) }))) })));
}
export const DSelect = React.forwardRef(Select);
//# sourceMappingURL=Select.js.map