import { freeze, produce } from 'immer';
import { isFunction, isUndefined } from 'lodash';
import { useState } from 'react';
import { useEventCallback } from '@react-devui/hooks';
export function useDValue(initialValue, value, onChange, deepCompare, formControlInject) {
    const [_value, setValue] = useState(initialValue);
    const currentValue = isUndefined(formControlInject) ? (isUndefined(value) ? _value : value) : formControlInject[0];
    const changeValue = useEventCallback((updater) => {
        const newValue = isFunction(updater) ? produce(currentValue, updater) : freeze(updater);
        const shouldUpdate = deepCompare ? !deepCompare(currentValue, newValue) : !Object.is(currentValue, newValue);
        if (shouldUpdate) {
            setValue(newValue);
            onChange === null || onChange === void 0 ? void 0 : onChange(newValue);
            if (!isUndefined(formControlInject)) {
                formControlInject[1](newValue);
            }
        }
        return newValue;
    });
    return [currentValue, changeValue];
}
//# sourceMappingURL=useDValue.js.map